#ifndef _PHORB_H
#define _PHORB_H

#include "php.h"
#include "zend_compile.h"
#include "php_globals.h"

#include <glib.h>
#include <orbit/orbit.h>
#include <sys/time.h>

#define CLASS_SEPARATOR		"."
#define IDL_CACHE_TIMEOUT	300
#define IDL_CACHE_CHECK		30

typedef struct _phorbIfaceInfo	phorbIfaceInfo;
typedef struct _phorbObject	phorbObject;
typedef struct _phorbIDL	phorbIDL;

struct _phorbIfaceInfo {
	ORBit_IInterface	*iface;
	GHashTable		*methods;
	GSList			*base;
};

struct _phorbIDL {
	gchar		*name;
	time_t		lastchanged;
	time_t		lastaccessed;
	time_t		lastchecked;
	GHashTable	*typecodes;
	GHashTable	*interfaces;
	GHashTable	*exceptions;
};

/* invocation.c */

void		phorb_ensure_orb(CORBA_Environment *ev);
void		phorb_shutdown_orb(void);
void		phorb_init_corba_class(zend_class_entry *objclass);
void		phorb_fini_corba_class(void);
int		phorb_register_destructor(void (*simple)(void *), void (*persistent)(void *));
gboolean	phorb_is_object(zval *val);
zval		*phorb_init_object(zval *res);
ulong		phorb_register_object(CORBA_Object obj);
CORBA_char	*phorb_get_object_repoid(CORBA_Object cobj, CORBA_Environment *ev);
void		phorb_invoke_method(INTERNAL_FUNCTION_PARAMETERS, zend_property_reference *props);
zval		phorb_get_attribute(zend_property_reference *props);
int		phorb_set_attribute(zend_property_reference *props, zval *value);
void		phorb_serialize_sleep(INTERNAL_FUNCTION_PARAMETERS);
void		phorb_serialize_wakeup(INTERNAL_FUNCTION_PARAMETERS);


/* typecode.c */

gboolean	phorb_is_typecode(zval *val);
zval		*phorb_init_typecode(zval *res);
ulong		phorb_register_typecode(CORBA_TypeCode tc);
void		phorb_init_typecode_class(zend_class_entry *tcclass);
void		phorb_fini_typecode_class(void);
zval		phorb_get_typecode_attribute(zend_property_reference *props);
int		phorb_set_typecode_attribute(zend_property_reference *props, zval *value);
void		phorb_serialize_typecode_sleep(INTERNAL_FUNCTION_PARAMETERS);
void		phorb_serialize_typecode_wakeup(INTERNAL_FUNCTION_PARAMETERS);
CORBA_TypeCode	phorb_find_typecode(phorbIDL *idl, const char *repoid);
void		phorb_store_typecode(phorbIDL *idl, CORBA_TypeCode type);
void		phorb_setup_typecodes(phorbIDL *idl);
void		phorb_invoke_typecode_method(INTERNAL_FUNCTION_PARAMETERS, zend_property_reference *props);
CORBA_long	phorb_enum_find_member(CORBA_TypeCode tc, zval *val);
CORBA_long	phorb_union_find_arm(CORBA_TypeCode tc, zval *discriminator);


/* interfaces.c */

phorbIfaceInfo	*phorb_init_interface(phorbIDL *idl, ORBit_IInterface *iface);
phorbIfaceInfo	*phorb_find_interface_description(phorbIDL *idl, const char *repoid);
void		phorb_free_interface_info(phorbIfaceInfo *info);
void		phorb_xref_interfaces(phorbIDL *idl);
ORBit_IMethod	*phorb_find_interface_method(phorbIfaceInfo *info, const char *name);


/* idl.c */

gboolean	phorb_load_idl_from_file(const char *file);
phorbIDL	*phorb_alloc_idl(const char *name);
void		phorb_free_idl(phorbIDL *idl);
void		phorb_check_idl_cache(void);

/* marshal.c */

gboolean	phorb_put_zval(gconstpointer *buf, CORBA_TypeCode tc, zval *val);


/* demarshal.c */

zval		*phorb_get_zval(gconstpointer *buf, CORBA_TypeCode tc);


/* exceptions.c */

void		phorb_throw(zval *value);
void		phorb_catch(void);
const char	*phorb_find_exception(phorbIDL *idl, const char *repoid);
void		phorb_init_exceptions(void);
void		phorb_shutdown_exceptions(void);
void		phorb_setup_exceptions(phorbIDL *idl);
zval		*phorb_system_except(const char *repoid, CORBA_unsigned_long minor, CORBA_completion_status status);
zval		*phorb_user_except(const char *repoid, zval *value);
int		phorb_reset_exception_counter(void);
void		phorb_exception_set_user_function(const char *fname);

#endif
